#ifndef EYES_CONFIG_HPP
#define EYES_CONFIG_HPP

#include <iostream>
#include <opencv2/opencv.hpp>
#include <stdexcept>

namespace eyetrack
{

/*!
** \brief struct eye_s
** Contient l'image d'un oeil, sa direction et le score de ressemblance avec une autre image
*/
struct eye_s
{
	eye_s() : _matchScore(0.0)
	{
	}

	/*!
	** \brief Constructeur
	** @param String filename	: nom de l'image à charger
	** @param Integer flags (0)	: flags d'ouverture compatible OpenCV
	*/
	eye_s(const std::string& filename, const int flags = 0) : _matchScore(0.0)
	{
		load(filename, flags);
	}

	/*!
	** \brief
	** Enregistre l'image en mémoire sur le disque dur
	** @param String filename : le chemin/nom du fichier à écrire
	** @return Boolean : true si l'image existe en mémoire et si l'enregistrement a réussi
	*/
	bool		write(const std::string& filename)
	{
		if (_mat.empty())
			return false;
		try
		{
			cv::imwrite(filename, _mat);
		}
		catch (const std::runtime_error& ex)
		{
			std::cerr << "Exception in eye_s::write(): " << ex.what() << std::endl;
			return false;
		}
		return true;
	}

	/*!
	** \brief
	** Charge l'image en mémoire
	** @param String filename : le fichier à utiliser
	** @param Integer flags : les flags d'ouverture compatible OpenCV
	** @return Boolean : true si le fichier existe et si son chargement a réussi
	*/
	bool		load(const std::string& filename, const int flags = 0)
	{
		_mat = cv::imread(filename, flags);
		return !_mat.empty();
	}

	cv::Mat		_mat;
	double		_matchScore;
	Direction	_dir;
};

/*!
** \brief Classe utilitaire pour lancer la comparaison entre deux images
*/
class eye_compare
{
public:
	/*!
	** \brief
	** Execute la comparaison entre deux images en utilisant la méthode CV_TM_CCOEFF_NORMED
	** @param struct Eye lhs : l'objet à comparer, ne subit de modification que s'il correspond mieux au modèle rhs
	** @param struct Eye rhs : le modèle de comparaison
	*/
	static void	exec_compare(eye_s& lhs, const eye_s& rhs)
	{
		if (lhs._mat.empty() || rhs._mat.empty())
			return ;

		cv::Mat		res;
		cv::Point	minloc, maxloc;
		double		minval, maxval = 0.0;

		try
		{
			cv::matchTemplate(lhs._mat, rhs._mat, res, CV_TM_CCOEFF_NORMED);
			cv::minMaxLoc(res, &minval, &maxval, &minloc, &maxloc);
		}
		catch (const cv::Exception& ex) {
			std::cerr << "Exception in template matching: " << ex.what() << std::endl;
		}
		if (lhs._matchScore < maxval) {
			lhs._dir = rhs._dir;
			lhs._matchScore = maxval;
		}
	}
};

}

#endif // !EYES_CONFIG_HPP