#include <sstream>
#include "eyes.h"
#include "eyetrack.h"

using namespace eyetrack;

const std::string	Eyes::DEFAULT_CASCADE = "/resources/haarcascades/haarcascade_eye_tree_eyeglasses.xml";
const qint64		Eyes::BLINK_TIME_IN_MSEC_TO_EXEC = 500; // Blink nature validated after ...msec

Eyes::Eyes(eyes_actions_callback& callback, const QString& folderPath) : ObjectDetect(), mCallback(callback), mFolderPath(folderPath)
{
	setClassifier(folderPath.toStdString() + DEFAULT_CASCADE);
	bool conf = load_config();
	if (conf == false)
		std::cout << "Invalid or incomplete eyes configuration, you will have to train your eyes!" << std::endl;
	else
		std::cout << "Eyes configuration OK!" << std::endl;
	mStartBlinkingTimer.start();
	mDirectionTimer.start();
}

Eyes::~Eyes()
{
}

void			Eyes::distinct_blinks()
{
	eyes_blink	which_blink;

	// Clignement des deux yeux
	if (mLeftEye._mat.empty() && mRightEye._mat.empty())
		which_blink = DOUBLE_BLINK;
	// Clignement de l'oeil gauche
	else if (mLeftEye._mat.empty() && !mRightEye._mat.empty())
		which_blink = BLINK_LEFT;
	// Clignement de l'oeil droit
	else if (!mLeftEye._mat.empty() && mRightEye._mat.empty())
		which_blink = BLINK_RIGHT;
	// Aucun
	else
		which_blink = NONE;

	mBlinks.push_back(which_blink);

	if (mStartBlinkingTimer.hasExpired(BLINK_TIME_IN_MSEC_TO_EXEC))
	{
		which_blink = mBlinks.star_value();
		if (which_blink != NONE)
			mCallback.emit_signal_blink(DOUBLE_BLINK);
		mBlinks.clear();
		mStartBlinkingTimer.restart();
	}
}

void		Eyes::distinct_eyes()
{
	// Pas de rgion d'intrt dans l'image, donc pas d'yeux dtects
	if (mROI.size() == 0)
	{
		mLeftEye._mat.release();
		mRightEye._mat.release();
	}

	// Sinon avec les deux yeux
	else if (mROI.size() == 2)
	{
		// On distingue les yeux droit et gauche
		if (mROI[0].x < mROI[1].x) {
			mLeftEye._mat = mInputFrame(mROI[1]);
			mRightEye._mat = mInputFrame(mROI[0]);
		}
		else {
			mLeftEye._mat = mInputFrame(mROI[0]);
			mRightEye._mat = mInputFrame(mROI[1]);
		}
	}

	// Ou alors un seul oeil
	else if (mROI.size() == 1)
	{
		// OPTIMIZATION REQUIRED
		if (mROI[0].x > mInputFrame.cols / 2.0) // Oeil gauche
		{
			mRightEye._mat.release();
			mLeftEye._mat = mInputFrame(mROI[0]);
		}
		else // Oeil droit
		{
			mLeftEye._mat.release();
			mRightEye._mat = mInputFrame(mROI[0]);
		}
	}
}

Direction				Eyes::lookAt()
{
	distinct_eyes();
	distinct_blinks();

	mLeftEye._dir = mRightEye._dir = UNKNOWN;
	mLeftEye._matchScore = mRightEye._matchScore = 0.0;

	foreach (const eye_s& tmp, mLeftEyeTemplates)
		eye_compare::exec_compare(mLeftEye, tmp);

	foreach (const eye_s& tmp, mRightEyeTemplates)
		eye_compare::exec_compare(mRightEye, tmp);

	if (mLeftEye._dir == mRightEye._dir)
		manageDirections(mLeftEye._dir);
	else if (mLeftEye._matchScore > mRightEye._matchScore)
		manageDirections(mLeftEye._dir);
	else
		manageDirections(mRightEye._dir);
	return mDirections.star_value();
}

void		Eyes::manageDirections(const Direction dir)
{
	/*if (mDirections.size() >= 10)
		mDirections.pop_front();*/
	static Direction pdir = UNKNOWN;

	mDirections.push_back(dir);
	if (mDirectionTimer.hasExpired(300))
	{
		Direction current = mDirections.star_value();

		if (current != pdir && mROI.size())
			mCallback.emit_signal_direction(mDirections.star_value());
		pdir = current;
		mDirections.clear();
		mDirectionTimer.restart();
	}
}

bool		Eyes::trainDirection(const Direction dir)
{
	if (mROI.size() != 2)
		return false;

	distinct_eyes();

	mLeftEye._dir = mRightEye._dir = dir;

	std::ostringstream filename;

	filename << mFolderPath.toStdString() << "/resources/eyes_lookat/LEFTEYE_" << dir << ".png";
	if (!mLeftEye.write(filename.str()))
		return false;

	filename.str("");
	filename << mFolderPath.toStdString() << "/resources/eyes_lookat/RIGHTEYE_" << dir << ".png";
	if (!mRightEye.write(filename.str()))
		return false;

	mLeftEyeTemplates.push_back(mLeftEye);
	mRightEyeTemplates.push_back(mRightEye);
	return true;
}

bool	Eyes::load_config()
{
	std::ostringstream	os;
	const std::string	folderPath(mFolderPath.toStdString() + "/resources/eyes_lookat/");

	mLeftEyeTemplates.clear();
	mRightEyeTemplates.clear();

	for (int i = 1; i < 6; ++i)
	{
		eye_s	left, right;

		os.str("");
		os << folderPath << "LEFTEYE_" << i << ".png";

		if (!left.load(os.str()))
			return false;

		os.str("");
		os << folderPath << "RIGHTEYE_" << i << ".png";

		if (!right.load(os.str()))
			return false;

		left._dir = right._dir = static_cast<Direction>(i);
		mLeftEyeTemplates.push_back(left);
		mRightEyeTemplates.push_back(right);
	}
	return true;
}
