#include <QMainWindow>
#include <QElapsedTimer>
#include <ctime>
#include "eyetrack.h"

using namespace eyetrack;

const double		EyeTrack::FRAME_WIDTH	= 800;
const double		EyeTrack::FRAME_HEIGHT	= 600;
const std::string	EyeTrack::WINDOW_NAME	= "EyeTrack";

EyeTrack::EyeTrack(eyes_actions_callback& callback, const QString& folderPath, QObject* parent) : QObject(parent), mCallback(callback)
{
	// Initialisation des membres
	mFace		= new ObjectDetect();
	mEyes		= new Eyes(callback, folderPath);

	// Assignement des proprits
	mCapture.set(CV_CAP_PROP_FRAME_WIDTH, FRAME_WIDTH);
	mCapture.set(CV_CAP_PROP_FRAME_HEIGHT, FRAME_HEIGHT);
	mFace->setClassifier(folderPath.toStdString() + "/resources/lbpcascade_frontalface.xml");

	// Ouverture de la fenetre de rendu
	//cv::namedWindow(WINDOW_NAME);
}

EyeTrack::~EyeTrack()
{
	//cv::destroyWindow(WINDOW_NAME);
	delete mFace;
	delete mEyes;
}

bool		EyeTrack::run_tracker()
{
	if (mCapture.open(0) == false)
	{
		mCallback.emit_signal_no_device();
		return false;
	}

	mOpened = true;

	trainingProcess();

	// Check si la configuration des yeux est ok
	if (mEyes->load_config() == true)
		track();
	// Sinon on lance la procdure d'entrainement
	else
	{
		trainingProcess();
		track();
	}
	mCapture.release();
	emit finished();
	return true;
}

void			EyeTrack::stop_tracker()
{
	//mMutex.lock();
	mOpened = false;
	//mMutex.unlock();
}

void			EyeTrack::track()
{
	Direction	dir, pdir = UNKNOWN;

	forever
	{
		if (!mOpened)
			break ;

		locateEyes(0);
		dir = mEyes->lookAt();
		/*if (dir != pdir)
			mCallback.emit_signal_direction(dir);*/
		pdir = dir;
	}
}

void				EyeTrack::trainingProcess()
{
	int				dir = 1;
	QElapsedTimer	timer;

	timer.start();
	mCallback.emit_signal_display_help(true);

	while (!timer.hasExpired(2000)) ;
	mCallback.emit_signal_direction(static_cast<Direction>(dir));
	timer.restart();
	forever
	{
		if (!mOpened || dir == 6)
			break ;
		
		locateEyes();
		if (mEyes->trainDirection(static_cast<Direction>(dir)))
		{
			if (timer.hasExpired(2000))
			{
				++dir;
				mCallback.emit_signal_direction(static_cast<Direction>(dir));
				timer.restart();
			}
		}
	}
	mCallback.emit_signal_display_help(false);
	mEyes->load_config();
}

void						EyeTrack::locateEyes(const qint64 timeout)
{
	static cv::Mat			frame, gray_frame;
	static QElapsedTimer	timer;
	static QElapsedTimer	eyes_problem_timer;
	static size_t			eyes_detected;
	static extdeque<bool>	false_positives;

	timer.restart();
	forever
	{
		try
		{
			mCapture >> frame;
			cv::cvtColor(frame, gray_frame, CV_BGR2GRAY);	
			mFace->setFrame(gray_frame);
			if (mFace->detect(1, cv::Size(FRAME_WIDTH / 4, FRAME_HEIGHT / 3)))
			{
				cv::Mat	faceROI = gray_frame(mFace->getROI()[0]);
				mEyes->setFrame(faceROI);
				eyes_detected = mEyes->detect(2, cv::Size((FRAME_WIDTH / 4) / 4, (FRAME_HEIGHT / 4) / 4));
				//showAll(frame);

				if (!eyes_detected)
					false_positives.push_back(false);
				else
					false_positives.push_back(true);

				if (eyes_problem_timer.hasExpired(2000))
				{
					mCallback.emit_signal_eyes_problem(!false_positives.star_value());
					false_positives.clear();
					eyes_problem_timer.restart();
				}

				if (eyes_detected == 2)
					break ;
			}
			if (timer.hasExpired(timeout))
				break ;
		}
		catch (const cv::Exception& e) {
			std::string error = "Exception caught : ";
			error += e.what();
			qDebug(error.c_str());
		}
	}
}

void		EyeTrack::showAll(cv::Mat& frame)
{
	if (frame.empty())
		return ;

	// On trace les contours de tte et d'yeux
	const std::vector<cv::Rect>&	faces = mFace->getROI();

	for(size_t i = 0; i < faces.size(); ++i)
	{
		cv::Point center(static_cast<int>(faces[i].x + faces[i].width * 0.5), static_cast<int>(faces[i].y + faces[i].height * 0.5));
		cv::ellipse(frame, center, cv::Size(static_cast<int>(faces[i].width * 0.5), static_cast<int>(faces[i].height * 0.5)), 0, 0, 360, cv::Scalar(255, 0, 255), 4, 8, 0);

		cv::Mat faceROI = frame(faces[i]);

		const std::vector<cv::Rect>& eyes = mEyes->getROI();

		for(size_t j = 0; j < eyes.size(); ++j)
		{
			cv::Point center(static_cast<int>(faces[i].x + eyes[j].x + eyes[j].width * 0.5), static_cast<int>(faces[i].y + eyes[j].y + eyes[j].height * 0.5));
			int radius = cvRound((eyes[j].width + eyes[j].height) * 0.25);
			cv::circle(frame, center, radius, cv::Scalar(255, 0, 0), 4, 8, 0);
		}
	}

	// Affichage de l'image
	cv::imshow(WINDOW_NAME, frame);
}